#include <stdlib.h>
#include <errno.h>
#include <string.h>

struct ident_ref
{
  char *name;
  size_t count;
  struct ident_ref *next;
};

static struct ident_ref *nametab;

int
ident_ref (char const *name, char const **refname)
{
  struct ident_ref *ref = NULL;
  
  if (!refname)
    return EINVAL;

  if (!name)
    {
      *refname = NULL;
      return 0;
    }
  
  if (nametab)
    {
      for (ref = nametab; ref; ref = ref->next)
	{
	  if (strcmp (ref->name, name) == 0)
	    break;
	}
    }

  if (!ref)
    {
      ref = malloc (sizeof (*ref));
      if (!ref)
	return -1;
      ref->name = strdup (name);
      if (!ref->name)
	{
	  free (ref);
	  return -1;
	}
      ref->count = 0;
      ref->next = nametab;
      nametab = ref;
    }
  
  ref->count++; 
  *refname = ref->name;
 
  return 0;
}

void
ident_deref (char const *name)
{
  struct ident_ref *ref, *prev;

  if (!name || !nametab)
    return;

  for (ref = nametab, prev = NULL; ref; prev = ref, ref = ref->next)
    {
      if (strcmp (ref->name, name) == 0)
	{
	  if (--ref->count == 0)
	    {
	      if (prev)
		prev->next = ref->next;
	      else
		nametab = ref->next;
	      free (ref->name);
	      free (ref);
	    }
	  break;
	}
    }
}
  

